Cluster <- function(var, weights = 1, nclusters = 4) {
  #
  # This function computes clusters by applying the k-mean method on the 
  # 2-dimensional input var (space, time). The first dimension can correspond
  # either to area-averages over a series of domains or to the grid-points 
  # from any grid (x,y), (x,z) ... re-organised in arrays for each time step
  #
  # Args:
  #  var : 2-dimensional matrice (time,space); the spatial dimension can 
  #        correspond either to area-averages over a series of domains or 
  #        to the grid-points from any grid (x,y), (x,z) ...
  #        re-organised in arrays for each time step
  #  weights : 1-dimensional array of weights corresponding to the areas 
  #            covered each domain or grid-cell of var; the length of 
  #            the weights array should equal the first dimension of var
  #  nclusters : number of clusters to be computed
  #
  # Returns:
  #   clusters : 2-dimensional matrice (spatial, nclusters) containing the
  #              centroides  
  #   pvalue   : 2-dimensional matrice (spatial, nclusters) containing the 
  #              pvalue of the centroide anomalies obtained through a t-test
  #              that accounts for the serial dependence of the data
  #   occurrences : 2-dimension matrice (nclusters, time) containing 1 (when
  #                 a given cluster occurs) or 0 (when it does not occur)
  #
  # History:
  #   
##############################################################################

library('fpc')

# General comment about the arguments
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# I did not put the number of iteration as an argument of the function because
# a criteria for convergence needs to be defined and this criteria would 
# determine whether or not the iterations have to be stopped. For example, you
# could compute the euclidian distance for each centroide between iteration n
# and iteration n+1 and sum these distance. Let's call this total distance TD.
# You would keep the list of distances between each iterations list (TD1, TD2,
# TD3 ...). When [ TD(n+1)-TD(n) ] < = 0.01 [TD(2)-TD(1) ], you would stop. 
# The factor 0.01 could be fine-tuned when you make tests.
# Unless you have another idea for a criteria ? Maybe they have something else
# in Michelangelu, I don't remember
# In any case, we need a criteria, it should not be trial and errors but a well-
# defined procedure.

# The 3 lines below should be moved to the Cluster function and here you would
# simply have a call to CLuster(bubi) for now without any output returned.
print(' ')
print('The K-means analysis with K=4')
print(' ')
print(kmeans.results<-kmeans(var,nclusters,iter.max=100,nstart=50))

# Function
# ~~~~~~~~~
# 
# Your function should start by a security check that the length of weights 
# equals the first dimension of var

# Then you should have a while loop to compute and recompute the clusters
# at each iteration you call the package you have choosen and you get the 
# centroids. From step 2, you compute the distance with previous centroides
# The cleaner way to do that is to use a CentroideDistance function that would
# be called at each iteration. When you reach the criteria you get out of the 
# loop and you have your centroides and their occurrences
#
# Finally you loop over the spatial point to apply a t-test and check the 
# significance of the centroide anomalies. You can call Eno to get the number 
# of independent data along time.
#
# The 3 matrices of ouputs are returned through a list.

  Cluster <- array(dim=c(length(kmeans.results$cluster)))
  Cluster[] <- kmeans.results$cluster
  # output
  Cluster
} 
